/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Copyright (c) 2009 Hewlett-Packard, Inc. All rights reserved.
 * Use is subject to license terms.
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */
package com.hp.opencall.jmsc.test.mandatory.functional;

import javax.media.mscontrol.MediaConfig;
import javax.media.mscontrol.Parameter;
import javax.media.mscontrol.Parameters;
import javax.media.mscontrol.mediagroup.MediaGroup;
import javax.media.mscontrol.mediagroup.Player;

import org.apache.log4j.Logger;

import com.hp.opencall.jmsc.test.AssertionIds;
import com.hp.opencall.jmsc.test.MediaSessionBase;

public class MediaConfigTest extends MediaSessionBase {
	static Logger log = Logger.getLogger(MediaConfigTest.class);
	
	public MediaConfigTest() {
		super();
	}
	public MediaConfigTest(String testMethod) {
		super(testMethod);
	}
	
	/**
	 * Create a MediaGroup with a customized MediaConfig.
	 * <br>The original MediaConfig is MediaGroup.PLAYER.
	 * <br>The Parameter Player.REPEAT_COUNT is changed in a customized MediaConfig.
	 * <br>This customized MediaConfig is used to create a MediaGroup, and the value is checked.
	 */
	@AssertionIds( ids = { "javadoc::MediaConfig::1" }, desc = "createCustomizedClone returns a MediaConfig with customized values")
	public void test_2_1_19_1_CustomizedClone() throws Exception {
		MediaConfig config = myMsControlFactory.getMediaConfig(MediaGroup.PLAYER);
		Parameter testedParam = Player.REPEAT_COUNT;
		Parameters custoParams = createParameters();
		Integer customValue = new Integer(7); // surely not the default value
		custoParams.put(testedParam, customValue);
		MediaConfig customConfig = config.createCustomizedClone(custoParams);
		Integer retrievedCustomValue = (Integer)myMediaSession.createMediaGroup(customConfig, Parameters.NO_PARAMETER)
					.getParameters(new Parameter[]{testedParam})
					.get(testedParam);
		log.info("Customized value: "+testedParam+"="+retrievedCustomValue);
		assertEquals("Retrieve wrong customized "+testedParam+" value: ", customValue, retrievedCustomValue);
	}	
}
