/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Copyright (c) 2009 Hewlett-Packard, Inc. All rights reserved.
 * Use is subject to license terms.
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */
package com.hp.opencall.jmsc.test.mandatory.functional;

import javax.media.mscontrol.MediaObject;
import javax.media.mscontrol.MediaSession;
import javax.media.mscontrol.Parameters;
import javax.media.mscontrol.mediagroup.MediaGroup;
import javax.media.mscontrol.mixer.MediaMixer;
import javax.media.mscontrol.mixer.MixerAdapter;
import javax.media.mscontrol.networkconnection.NetworkConnection;

import org.apache.log4j.Logger;

import com.hp.opencall.jmsc.test.AssertionIds;
import com.hp.opencall.jmsc.test.MediaSessionBase;

public class MediaObjectIdTest extends MediaSessionBase {
	static Logger log = Logger.getLogger(MediaObjectIdTest.class);
	
	public MediaObjectIdTest() {
		super();
	}
	public MediaObjectIdTest(String testMethod) {
		super(testMethod);
	}

	/**
	 * Check that different MediaSessions can host MediaObjects with the same MEDIAOBJECT_ID.
	 */
	@AssertionIds( ids = {"javadoc::MediaObject::4"})
	public void test_2_1_20_PerSessionId() throws Exception {
		MediaSession ms1 = createObjectHierarchy();
		MediaSession ms2 = createObjectHierarchy();
		ms1.release();
		ms2.release();
	}
	
	private MediaSession createObjectHierarchy() throws Exception {
		MediaSession ms = myMsControlFactory.createMediaSession();
		assertNotNull(ms);
		Parameters params = ms.createParameters();
		params.put(MediaObject.MEDIAOBJECT_ID, "MediaGroup-1");
		MediaGroup mg = ms.createMediaGroup(MediaGroup.PLAYER, params);
		assertNotNull(mg);
		params.put(MediaObject.MEDIAOBJECT_ID, "NetworkConnection-1");
		NetworkConnection nc = ms.createNetworkConnection(NetworkConnection.BASIC, params);
		assertNotNull(nc);
		params.put(MediaObject.MEDIAOBJECT_ID, "MediaMixer-1");
		MediaMixer mx = ms.createMediaMixer(MediaMixer.AUDIO, params);
		assertNotNull(mx);
		params.put(MediaObject.MEDIAOBJECT_ID, "MixerAdapter-1");
		MixerAdapter ma = mx.createMixerAdapter(MixerAdapter.DTMF_CLAMP, params);
		assertNotNull(ma);
		return ms;
	}
}
