/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Copyright (c) 2009 Hewlett-Packard, Inc. All rights reserved.
 * Use is subject to license terms.
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */
package com.hp.opencall.jmsc.test.mandatory.functional;

import java.util.HashSet;
import java.util.Iterator;

import javax.media.mscontrol.MediaConfig;
import javax.media.mscontrol.MediaObject;
import javax.media.mscontrol.Parameters;
import javax.media.mscontrol.mediagroup.MediaGroup;
import javax.media.mscontrol.mixer.MediaMixer;
import javax.media.mscontrol.networkconnection.NetworkConnection;
import javax.media.mscontrol.vxml.VxmlDialog;

import org.apache.log4j.Logger;

import com.hp.opencall.jmsc.test.AssertionIds;
import com.hp.opencall.jmsc.test.MediaSessionBase;

public class MediaSessionTest extends MediaSessionBase {

	static Logger log = Logger.getLogger(MediaSessionTest.class);
	
	public MediaSessionTest() {
		super();
	}
	public MediaSessionTest(String testMethod) {
		super(testMethod);
	}
	
	/**
	 * Create NetworkConnection with "createNetworkConnection(NetworkConnection.BASIC)".
	 */
	@AssertionIds( ids = { "javadoc::MediaSession::1" }, desc = "createNetworkConnection(NetworkConnection.BASIC) returns a NetworkConnection")
	public void test_2_1_11_1_NetworkConnectionContainer() throws Exception {
		NetworkConnection medObj = myMediaSession.createNetworkConnection(NetworkConnection.BASIC);
		assertNotNull("Create "+NetworkConnection.BASIC+" failed", medObj);
	}	
	/**
	 * Create NetworkConnection with "createNetworkConnection(NetworkConnection.BASIC, Parameters)".
	 */	
	@AssertionIds( ids = { "javadoc::MediaSession::2" }, desc = "createNetworkConnection(NetworkConnection.BASIC, Parameters) returns a NetworkConnection")
	public void test_2_1_11_2_NetworkConnectionContainer() throws Exception {
		String id = "testID";
		Parameters params = myMediaSession.createParameters();
		params.put(MediaObject.MEDIAOBJECT_ID, id);
		NetworkConnection medObj = myMediaSession.createNetworkConnection(NetworkConnection.BASIC, params);
		assertNotNull("Create "+NetworkConnection.BASIC+" failed", medObj);
		String uri = medObj.getURI().toString();
		assertTrue("URI does not contain MEDIAOBJECT_ID \""+id+"\": "+uri, uri.contains(id));
	}	
	/**
	 * Create NetworkConnection with "createNetworkConnection(MediaConfig, Parameters)".
	 */	
	@AssertionIds( ids = { "javadoc::MediaSession::3" }, desc = "createNetworkConnection(MediaConfig, Parameters) returns a NetworkConnection")
	public void test_2_1_11_3_NetworkConnectionContainer() throws Exception {
		String id = "testID";
		MediaConfig config = myMsControlFactory.getMediaConfig(NetworkConnection.BASIC);
		Parameters params = myMediaSession.createParameters();
		params.put(MediaObject.MEDIAOBJECT_ID, id);
		NetworkConnection medObj = myMediaSession.createNetworkConnection(config, params);
		assertNotNull("Create "+NetworkConnection.BASIC+" failed", medObj);
		String uri = medObj.getURI().toString();
		assertTrue("URI does not contain MEDIAOBJECT_ID \""+id+"\": "+uri, uri.contains(id));
	}
	
	/**
	 * Create MediaGroup with "createMediaGroup(MediaGroup.BASIC)".
	 */
	@AssertionIds( ids = { "javadoc::MediaSession::4" }, desc = "createMediaGroup(MediaGroup.BASIC) returns a MediaGroup")
	public void test_2_1_11_4_MediaGroupContainer() throws Exception {
		MediaGroup medObj = myMediaSession.createMediaGroup(MediaGroup.PLAYER);
		assertNotNull("Create "+MediaGroup.PLAYER+" failed", medObj);
	}	
	/**
	 * Create MediaGroup with "createMediaGroup(MediaGroup.PLAYER, Parameters)".
	 */	
	@AssertionIds( ids = { "javadoc::MediaSession::5" }, desc = "createMediaGroup(MediaGroup.PLAYER, Parameters) returns a MediaGroup")
	public void test_2_1_11_5_MediaGroupContainer() throws Exception {
		String id = "testID";
		Parameters params = myMediaSession.createParameters();
		params.put(MediaObject.MEDIAOBJECT_ID, id);
		MediaGroup medObj = myMediaSession.createMediaGroup(MediaGroup.PLAYER, params);
		assertNotNull("Create "+MediaGroup.PLAYER+" failed", medObj);
		String uri = medObj.getURI().toString();
		assertTrue("URI does not contain MEDIAOBJECT_ID \""+id+"\": "+uri, uri.contains(id));
	}	
	/**
	 * Create MediaGroup with "createMediaGroup(MediaConfig, Parameters)".
	 */	
	@AssertionIds( ids = { "javadoc::MediaSession::6" }, desc = "createMediaGroup(MediaConfig, Parameters) returns a MediaGroup")
	public void test_2_1_11_6_MediaGroupContainer() throws Exception {
		String id = "testID";
		MediaConfig config = myMsControlFactory.getMediaConfig(MediaGroup.PLAYER);
		Parameters params = myMediaSession.createParameters();
		params.put(MediaObject.MEDIAOBJECT_ID, id);
		MediaGroup medObj = myMediaSession.createMediaGroup(config, params);
		assertNotNull("Create "+MediaGroup.PLAYER+" failed", medObj);
		String uri = medObj.getURI().toString();
		assertTrue("URI does not contain MEDIAOBJECT_ID \""+id+"\": "+uri, uri.contains(id));
	}
	
    
	/**
	 * Create MediaMixer with "createMediaMixer(MediaMixer.AUDIO)".
	 */
	@AssertionIds( ids = { "javadoc::MediaSession::7" }, desc = "createMediaMixer(MediaMixer.AUDIO) returns a MediaMixer")
	public void test_2_1_11_7_MediaMixerContainer() throws Exception {
		MediaMixer medObj = myMediaSession.createMediaMixer(MediaMixer.AUDIO);
		assertNotNull("Create "+MediaMixer.AUDIO+" failed", medObj);
	}	
	/**
	 * Create MediaMixer with "createMediaMixer(MediaMixer.AUDIO, Parameters)".
	 */	
	@AssertionIds( ids = { "javadoc::MediaSession::8" }, desc = "createMediaMixer(MediaMixer.AUDIO, Parameters) returns a MediaMixer")
	public void test_2_1_11_8_MediaMixerContainer() throws Exception {
		String id = "testID";
		Parameters params = myMediaSession.createParameters();
		params.put(MediaObject.MEDIAOBJECT_ID, id);
		MediaMixer medObj = myMediaSession.createMediaMixer(MediaMixer.AUDIO, params);
		assertNotNull("Create "+MediaMixer.AUDIO+" failed", medObj);
		String uri = medObj.getURI().toString();
		assertTrue("URI does not contain MEDIAOBJECT_ID \""+id+"\": "+uri, uri.contains(id));
	}	
	/**
	 * Create MediaMixer with "createMediaMixer(MediaConfig, Parameters)".
	 */	
	@AssertionIds( ids = { "javadoc::MediaSession::9" }, desc = "createMediaMixer(MediaConfig, Parameters) returns a MediaMixer")
	public void test_2_1_11_9_MediaMixerContainer() throws Exception {
		String id = "testID";
		MediaConfig config = myMsControlFactory.getMediaConfig(MediaMixer.AUDIO);
		Parameters params = myMediaSession.createParameters();
		params.put(MediaObject.MEDIAOBJECT_ID, id);
		MediaMixer medObj = myMediaSession.createMediaMixer(config, params);
		assertNotNull("Create "+MediaMixer.AUDIO+" failed", medObj);
		String uri = medObj.getURI().toString();
		assertTrue("URI does not contain MEDIAOBJECT_ID \""+id+"\": "+uri, uri.contains(id));
	}
    
	/**
	 * Create VxmlDialog with "createVxmlDialog(Parameters)".
	 */	
	@AssertionIds( ids = { "javadoc::MediaSession::10" }, desc = "createVxmlDialog(Parameters) returns a VxmlDialog or throws exception")
	public void test_2_1_11_10_CreateVxmlDialog() throws Exception {
		String id = "testID";
		Parameters params = myMediaSession.createParameters();
		params.put(MediaObject.MEDIAOBJECT_ID, id);
		VxmlDialog medObj = null;
		try {
			medObj = myMediaSession.createVxmlDialog(params);
		} catch(Exception e) {
			log.info("Cannot create VxmlDialog - assuming that VxmlDialog is not supported by this implementation - terminating");
			return;
		}
		assertNotNull("Create VxmlDialog failed", medObj);
		String uri = medObj.getURI().toString();
		assertTrue("URI does not contain MEDIAOBJECT_ID \""+id+"\": "+uri, uri.contains(id));
	}
	/**
	 * Test set/getAttribute of a MediaSession.
	 */
	@AssertionIds( ids = { "javadoc::MediaSession::11" }, desc = "MediaSession attributes must be settable and gettable")
	public void test_2_1_11_11_SetGetAttribute() throws Exception {
		Integer attrib = new Integer(11);
		String attribName = "test attribute name";
		myMediaSession.setAttribute(attribName, attrib);
		assertEquals("Retrieved bad attribute: ", attrib, myMediaSession.getAttribute(attribName));
	}
	/**
	 * Test removeAttribute of a MediaSession.
	 */
	@AssertionIds( ids = { "javadoc::MediaSession::12" }, desc = "MediaSession attributes must be removable")
	public void test_2_1_11_12_RemoveAttribute() throws Exception {
		Integer attrib = new Integer(11);
		String attribName = "test attribute name";
		myMediaSession.setAttribute(attribName, attrib);
		myMediaSession.removeAttribute(attribName);
		assertNull("Attribute not removed", myMediaSession.getAttribute(attribName));
	}
	/**
	 * Test getAttributeNames of a MediaSession.
	 * Verify that the Set of Strings returned by {@link javax.media.mscontrol.MediaSession#getAttributeNames()}
	 * is actually the same as the Set of set names.
	 */
	@AssertionIds( ids = { "javadoc::MediaSession::13" }, desc = "MediaSession attributes names must be retrievable")
	public void test_2_1_11_13_GetAttributeNames() throws Exception {
		HashSet<String> names = new HashSet<String>();
		names.add("attr1");
		names.add("attr2");
		names.add("attr3");
		for (String name: names)
			myMediaSession.setAttribute(name, name+" value");
		Iterator<String> iter = myMediaSession.getAttributeNames();
		HashSet<String> gotNames = new HashSet<String>();
		while(iter.hasNext())
			gotNames.add(iter.next());
		assertEquals("Retrieved different attribute names", names, gotNames);
	}
}
