/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Copyright (c) 2008 Hewlett-Packard, Inc. All rights reserved.
 * Use is subject to license terms.
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */
package com.hp.opencall.jmsc.test.mandatory.functional.mixer;

import javax.media.mscontrol.Parameters;
import javax.media.mscontrol.join.Joinable.Direction;
import javax.media.mscontrol.mediagroup.signals.SignalDetector;
import javax.media.mscontrol.mediagroup.signals.SignalDetectorEvent;
import javax.media.mscontrol.mixer.MediaMixer;
import javax.media.mscontrol.networkconnection.NetworkConnection;
import javax.media.mscontrol.resource.RTC;

import org.apache.log4j.Logger;

import com.hp.opencall.jmsc.test.AssertionIds;
import com.hp.opencall.jmsc.test.MediaSessionBase;
import com.hp.opencall.jmsc.test.SipPhoneEmulator;

/**
 * Explores the simple uses of a Mixer.
 */
public class MixerTest extends MediaSessionBase {
	static Logger log = Logger.getLogger(MixerTest.class);

	public MixerTest() {
		super();
	}
	public MixerTest(String testMethod) {
		super(testMethod);
	}
	
	/**
	 * Basic test: create a MediaMixer and verify that it is not null.
	 * <br>The mixer is explicitly released at the end.
	 */
	@AssertionIds( ids = {"javadoc::MediaMixer::4" }, desc = "Can create a MediaMixer with Configuration AUDIO")
	public void test_2_1_5_1_CreateMixer() throws Exception {
		MediaMixer myMixer = myMediaSession.createMediaMixer(MediaMixer.AUDIO);
		
		if (isTrue("manual-mode"))
			promptUser("Mixer is created, hit any key to continue: ", true);
		
		assertNotNull(myMixer);
		myMixer.release(); // Anticipating on TestBase.release() : testing this also		
	}

	/**
	 * Create a Mixer, a NetworkConnection, and join them DUPLEX.
	 */
	public void test_2_1_5_2_JoinNetworkConnectionToMixer() throws Exception {
		final MediaMixer myMixer = myMediaSession.createMediaMixer(MediaMixer.AUDIO);
		NetworkConnection myNC = setupaNC();
		myMixer.join(Direction.DUPLEX, myNC);
	}
	/**
	 * Unjoin a NetworkConnection from a Mixer.
	 * <br>They were previously joined DUPLEX.
	 */
	public void test_2_1_5_3_UnjoinNetworkConnection() throws Exception {
		final MediaMixer myMixer = myMediaSession.createMediaMixer(MediaMixer.AUDIO);
		NetworkConnection myNC = setupaNC();
		myMixer.join(Direction.DUPLEX, myNC);
		myMixer.unjoin(myNC);
	}
	
	public void test_2_1_5_4_DTMFThroughMixer()  throws Exception {
		final SipPhoneEmulator adam = new SipPhoneEmulator(this, "Adam");
		final SipPhoneEmulator paul = new SipPhoneEmulator(this, "Paul");
		final MediaMixer myMixer = myMediaSession.createMediaMixer(MediaMixer.AUDIO);
		adam.myJoinPoint.join(Direction.DUPLEX, myMixer);
		paul.myJoinPoint.join(Direction.DUPLEX, myMixer);
		adam.setup(true, true);
		paul.setup(true, true);
		SignalDetector detector = paul.myMG.getSignalDetector();
		final EventWaiter<SignalDetectorEvent> sdEW = new EventWaiter<SignalDetectorEvent>(detector, log);
		detector.receiveSignals(1, SignalDetector.NO_PATTERN, RTC.NO_RTC, Parameters.NO_PARAMETER);
		adam.playPrompt(getProperty("testcase.dtmf1"));
		SignalDetectorEvent sdEvt = sdEW.waitForOKEvent(5000);
		assertEquals("Bad DTMF received", "1", sdEvt.getSignalString());	
	}
}
