/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Copyright (c) 2008 Hewlett-Packard, Inc. All rights reserved.
 * Use is subject to license terms.
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */
package com.hp.opencall.jmsc.test.mandatory.functional.vxml;

import java.net.URL;

import javax.media.mscontrol.Parameters;
import javax.media.mscontrol.join.Joinable.Direction;
import javax.media.mscontrol.networkconnection.NetworkConnection;
import javax.media.mscontrol.vxml.VxmlDialog;
import javax.media.mscontrol.vxml.VxmlDialogEvent;

import org.apache.log4j.Logger;

import com.hp.opencall.jmsc.test.MediaSessionBase;

/**
 * Basic testing of a VxmlDialog - Accept unsupported behaviour.
 * <br>Implementations not supporting voicexml will throw an exception on the
 * VxmlDialog creation, and this is accepted by the testcase.
 */
public class VxmlTest extends MediaSessionBase {
	static Logger log = Logger.getLogger(VxmlTest.class);

	public VxmlTest() {
		super();
	}
	public VxmlTest(String testMethod) {
		super(testMethod);
	}

	/**
	 * Basic testing of a VxmlDialog - Accept "unsupported" response.
	 * <p>Create, prepare and start a VxmlDialog joined to a NetworkConnection.
	 * <br>Implementations not supporting voicexml will throw an early exception on the
	 * VxmlDialog creation, and the testcase will terminate with success in that case.
	 * <br>Include calls to prepare(), processSdpOffer(SDPoffer), start().
	 * <br>The events {@link VxmlDialogEvent#PREPARED},
	 * {@link VxmlDialogEvent#STARTED} and {@link VxmlDialogEvent#EXITED} are checked.
	 */
	public void test_2_1_19_1_VxmlDialog() throws Exception {
		final NetworkConnection myNC = myMediaSession.createNetworkConnection(NetworkConnection.BASIC);
		final VxmlDialog myDialog;
		try {
			myDialog = myMediaSession.createVxmlDialog(Parameters.NO_PARAMETER);		
		} catch(Exception e) {
			log.warn("Cannot create VxmlDialog - assuming that VxmlDialog is not supported by this implementation - terminating");
			return;
		}
		myNC.join(Direction.DUPLEX, myDialog);
		
		EventWaiter<VxmlDialogEvent> vxW = new EventWaiter<VxmlDialogEvent>(myDialog, log);
		log.debug("myDialog.prepare(new URL("+getProperty("testcase.vxmlscript")+"), Parameters.NO_PARAMETER, null);");
		myDialog.prepare(new URL(getProperty("testcase.vxmlscript")), Parameters.NO_PARAMETER, null);
		log.debug("Setup NetworkConnection");
		setupNC(myNC);
		vxW.waitForEventID(VxmlDialogEvent.PREPARED);
		
		myDialog.start(null);
		vxW.waitForEventID(VxmlDialogEvent.STARTED);
		
		vxW.waitForEventID(VxmlDialogEvent.EXITED);
	}

}
