/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Copyright (c) 2008-2009 Hewlett-Packard, Inc. All rights reserved.
 * Use is subject to license terms.
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */
package com.hp.opencall.jmsc.test.mandatory.robustness.MSFaultTestServer;

import java.io.FileInputStream;
import java.io.InputStream;
import java.text.ParseException;
import java.util.Properties;

import javax.sip.ClientTransaction;
import javax.sip.Dialog;
import javax.sip.DialogTerminatedEvent;
import javax.sip.IOExceptionEvent;
import javax.sip.InvalidArgumentException;
import javax.sip.ListeningPoint;
import javax.sip.PeerUnavailableException;
import javax.sip.RequestEvent;
import javax.sip.ResponseEvent;
import javax.sip.ServerTransaction;
import javax.sip.SipException;
import javax.sip.SipFactory;
import javax.sip.SipListener;
import javax.sip.SipProvider;
import javax.sip.SipStack;
import javax.sip.Transaction;
import javax.sip.TransactionTerminatedEvent;
import javax.sip.address.Address;
import javax.sip.address.AddressFactory;
import javax.sip.header.ContactHeader;
import javax.sip.header.ContentTypeHeader;
import javax.sip.header.HeaderFactory;
import javax.sip.header.ToHeader;
import javax.sip.message.MessageFactory;
import javax.sip.message.Request;
import javax.sip.message.Response;


public class SIPServer_Base implements SipListener{


	protected static AddressFactory addressFactory;

	protected static MessageFactory messageFactory;

	protected static HeaderFactory headerFactory;

	protected static SipStack sipStack;

	protected SipProvider sipProvider;

	protected String fromIp;

	protected int fromPort;

	protected String mssdp;

	protected Dialog dialog;

	public static final boolean callerSendsBye = true;

	protected static final String usageString = "java "
		+ "examples.shootist.Shootist \n"
		+ ">>>> is your class path set to the root?";

	private static void usage() {
		System.out.println(usageString);
		System.exit(0);

	}


	public void processRequest(RequestEvent requestEvent) {
		Request request = requestEvent.getRequest();
		ServerTransaction serverTransactionId = requestEvent
		.getServerTransaction();

		System.out.println("\n\nRequest " + request.getMethod()
				+ " received at " + sipStack.getStackName()
				+ " with server transaction id " + serverTransactionId);

		if (request.getMethod().equals(Request.INVITE)) {
			processInvite(requestEvent, serverTransactionId);
		} else if (request.getMethod().equals(Request.ACK)) {
			processAck(requestEvent, serverTransactionId);
		} else if (request.getMethod().equals(Request.BYE)) {
			processBye(requestEvent, serverTransactionId);
		} else if (request.getMethod().equals(Request.CANCEL)) {

		} else {
			try {
				serverTransactionId.sendResponse( messageFactory.createResponse( 202, request ) );

				// send one back
				SipProvider prov = (SipProvider) requestEvent.getSource();
				Request refer = requestEvent.getDialog().createRequest("REFER");
				requestEvent.getDialog().sendRequest( prov.getNewClientTransaction(refer) );				

			} catch (SipException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (InvalidArgumentException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (ParseException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}

	}

	public void processResponse(ResponseEvent responseEvent) {
	}

	/**
	 * Process the ACK request. Send the bye and complete the call flow.
	 */
	public void processAck(RequestEvent requestEvent,
			ServerTransaction serverTransaction) {
		try {
			System.out.println("MS: got an ACK! ");
			System.out.println("Dialog State = " + dialog.getState());
			SipProvider provider = (SipProvider) requestEvent.getSource();
			if (!callerSendsBye) {
				Request byeRequest = dialog.createRequest(Request.BYE);
				ClientTransaction ct = provider
				.getNewClientTransaction(byeRequest);
				dialog.sendRequest(ct);
			}
		} catch (Exception ex) {
			ex.printStackTrace();
		}

	}

	/**
	 * Process the invite request.
	 */
	public void processInvite(RequestEvent requestEvent,
			ServerTransaction serverTransaction) {
		SipProvider sipProvider = (SipProvider) requestEvent.getSource();
		Request request = requestEvent.getRequest();
		try {
			System.out.println("MS: got an Invite sending Trying");
			// System.out.println("shootme: " + request);
			Response response = messageFactory.createResponse(Response.RINGING,
					request);
			ServerTransaction st = requestEvent.getServerTransaction();

			if (st == null) {
				st = sipProvider.getNewServerTransaction(request);
			}
			dialog = st.getDialog();

			st.sendResponse(response);

			Response resp = messageFactory.createResponse(Response.OK,
					request);
			Address address = addressFactory.createAddress("MS <sip:"
					+ fromIp + ":" + fromPort + ">");
			ContactHeader contactHeader = headerFactory
			.createContactHeader(address);
			resp.addHeader(contactHeader);
			ToHeader toHeader = (ToHeader) resp.getHeader(ToHeader.NAME);
			toHeader.setTag("4321"); // Application is supposed to set.
			resp.addHeader(contactHeader);
			String sdp = "v=0\no=acomment 123456 654321 IN IP4 127.0.0.1\nt=0 0\nm=audio 1234 RTP/AVP  0 101 \nc=IN IP4 127.0.0.1\na=rtpmap:0 PCMU/8000\na=rtpmap:101 telephone-event/8000/1\n";
			ContentTypeHeader cth = headerFactory.createContentTypeHeader("application", "sdp");
			resp.setContent(sdp.getBytes(),cth );
			st.sendResponse(resp);
			System.out.println("Sent 200 response for INVITE");
		} catch (Exception ex) {
			ex.printStackTrace();
			System.exit(0);
		}
	}


	/**
	 * Process the bye request.
	 */
	public void processBye(RequestEvent requestEvent, ServerTransaction serverTransactionId) {
		Request request = requestEvent.getRequest();
		Dialog dialog = requestEvent.getDialog();
		System.out.println("local party = " + dialog.getLocalParty());
		try {
			System.out.println("MS:  got a bye sending OK.");
			Response response = messageFactory.createResponse(200, request);
			serverTransactionId.sendResponse(response);
			System.out.println("Dialog State is "
					+ serverTransactionId.getDialog().getState());

		} catch (Exception ex) {
			ex.printStackTrace();
			System.exit(0);

		}
	}


	public void processTimeout(javax.sip.TimeoutEvent timeoutEvent) {
		Transaction transaction;
		if (timeoutEvent.isServerTransaction()) {
			transaction = timeoutEvent.getServerTransaction();
		} else {
			transaction = timeoutEvent.getClientTransaction();
		}
		System.out.println("state = " + transaction.getState());
		System.out.println("dialog = " + transaction.getDialog());
		System.out.println("dialogState = "
				+ transaction.getDialog().getState());
		System.out.println("Transaction Time out");
	}

	public void init() {
		SipFactory sipFactory = null;
		sipStack = null;
		sipFactory = SipFactory.getInstance();
		sipFactory.setPathName("gov.nist");
		Properties properties = new Properties();
		try {
			loadProps("testms.properties", properties, false);
		} catch (Exception e) {
			System.exit(1);
		}
		fromIp = properties.getProperty("mscontrolRI.javax.sip.ip");
		fromPort = Integer.valueOf(properties.getProperty("mscontrolRI.javax.sip.port"));
		mssdp = properties.getProperty("testcase.MSSDP");
		if(!(mssdp!=null&&mssdp.length()>0)) {
			mssdp = "v=0\no=acomment 123456 654321 IN IP4 16.16.230.57\nt=0 0\nm=audio 1234 RTP/AVP  0 101 \nc=IN IP4 16.16.230.57\na=rtpmap:0 PCMU/8000\na=rtpmap:101 telephone-event/8000/1\n";
		}
		properties.setProperty("javax.sip.STACK_NAME", "TEST-MS");
		// You need 16 for logging traces. 32 for debug + traces.
		// Your code will limp at 32 but it is best for debugging.
		properties.setProperty("gov.nist.javax.sip.TRACE_LEVEL", "32");
		properties.setProperty("gov.nist.javax.sip.DEBUG_LOG",
		"mstest.txt");
		properties.setProperty("gov.nist.javax.sip.SERVER_LOG",
		"mstest.txt");

		try {
			// Create SipStack object
			sipStack = sipFactory.createSipStack(properties);
			System.out.println("sipStack = " + sipStack);
		} catch (PeerUnavailableException e) {
			e.printStackTrace();
			System.err.println(e.getMessage());
			if (e.getCause() != null)
				e.getCause().printStackTrace();
			System.exit(0);
		}

		try {
			headerFactory = sipFactory.createHeaderFactory();
			addressFactory = sipFactory.createAddressFactory();
			messageFactory = sipFactory.createMessageFactory();
			ListeningPoint lp = sipStack.createListeningPoint(fromIp,
					fromPort, "udp");

			SIPServer_Base listener = this;

			sipProvider = sipStack.createSipProvider(lp);
			System.out.println("udp provider " + sipProvider);
			sipProvider.addSipListener(listener);

		} catch (Exception ex) {
			System.out.println(ex.getMessage());
			ex.printStackTrace();
			usage();
		}

	}

	public static void main(String args[]) {
		new SIPServer_Base().init();
	}

	public void processIOException(IOExceptionEvent exceptionEvent) {
		System.out.println("IOException");

	}

	public void processTransactionTerminated(
			TransactionTerminatedEvent transactionTerminatedEvent) {
		if (transactionTerminatedEvent.isServerTransaction())
			System.out.println("Transaction terminated event recieved"
					+ transactionTerminatedEvent.getServerTransaction());
		else
			System.out.println("Transaction terminated "
					+ transactionTerminatedEvent.getClientTransaction());

	}

	public void processDialogTerminated(
			DialogTerminatedEvent dialogTerminatedEvent) {
		System.out.println("Dialog terminated event recieved");
		Dialog d = dialogTerminatedEvent.getDialog();
		System.out.println("Local Party = " + d.getLocalParty());

	}

	private boolean loadProps(String fileName, Properties target, boolean allowFailure) throws Exception {
		// InputStream propStream = getClass().getResourceAsStream(fileName);
		InputStream propStream = null;
		try {
			propStream = new FileInputStream(fileName);
		} catch (Exception e) {
		}
		if (propStream == null) {
			if (! allowFailure) {
				System.exit(1);
			}
			return false;
		}
		target.load(propStream);
		return true;
	}

}
