/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Copyright (c) 2008-2009 Hewlett-Packard, Inc. All rights reserved.
 * Use is subject to license terms.
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */
package com.hp.opencall.jmsc.test.mandatory.robustness;

import javax.media.mscontrol.Configuration;
import javax.media.mscontrol.MsControlException;
import javax.media.mscontrol.mediagroup.MediaGroup;
import javax.media.mscontrol.mixer.MediaMixer;
import javax.media.mscontrol.mixer.MixerAdapter;

import org.apache.log4j.Logger;

import com.hp.opencall.jmsc.test.MediaSessionBase;

/**
 * Testcases involving null API call arguments.
 */
public class NullReferenceTest  extends MediaSessionBase {
	static Logger log = Logger.getLogger(NullReferenceTest.class);
	
    /**
     * 
     * Create MediaContainer object with null MediaConfig object we expected a
     * NullPointException
     * 
     * @throws Exception
     */
    public void test_2_2_6_1_CreateMediaObjectWithNullConfig() throws Exception {
		try {
		    myMediaSession.createMediaGroup(null);
		    fail("No exception is thrown when creating continer wiht null parameter.");
		} catch (MsControlException e) {
			log.debug("Expected MSControl Exception is thrown when creating MediaGroup with null Configuration.");
		}
		try {
		    myMediaSession.createNetworkConnection(null);
		    fail("No exception is thrown when creating continer wiht null parameter.");
		} catch (MsControlException e) {
		    log.debug("Expected MSControl Exception is thrown when creating NetworkConnection with null parameter.");
		}
		try {
		    myMediaSession.createMediaGroup((Configuration<MediaGroup>)null, createParameters());
		    fail("No exception is thrown when creating continer with null parameter.");
		} catch (MsControlException e) {
		    log.debug("Expected MSControl Exception is thrown when creating Mediagroup with null Configuration and Parameter.");
		}
    }

    
    /**
     * If create MixerAdapter is called with a null argument, an exception should
     * be thrown.
     * @throws Exception
     */
    public void test_2_2_6_2_CreateMixerAdapterWithNullParams() throws Exception {
		MediaMixer mixer = myMediaSession.createMediaMixer(MediaMixer.AUDIO);
		assertNotNull(mixer);
		// null MixerAdapter config
		MixerAdapter ma;
		try {
			ma = mixer.createMixerAdapter((Configuration<MixerAdapter>)null);
			fail("Created MixerAdapter with null parameter.");
			ma.release();
		} catch (MsControlException e) {
			log.debug("Expected MSControl Exception is thrown by creating MixerAdapter wiht null config.");
		}	
		mixer.release();
    }

}
