/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Copyright (c) 2008-2009 Hewlett-Packard, Inc. All rights reserved.
 * Use is subject to license terms.
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */
package com.hp.opencall.jmsc.test.mandatory.robustness;

import java.net.URI;

import javax.media.mscontrol.Parameter;
import javax.media.mscontrol.Parameters;
import javax.media.mscontrol.mediagroup.MediaGroup;
import javax.media.mscontrol.mediagroup.Recorder;
import javax.media.mscontrol.mediagroup.RecorderEvent;
import javax.media.mscontrol.mediagroup.signals.SignalDetector;
import javax.media.mscontrol.mediagroup.signals.SignalDetectorEvent;
import javax.media.mscontrol.resource.RTC;

import org.apache.log4j.Logger;

import com.hp.opencall.jmsc.test.AssertionIds;
import com.hp.opencall.jmsc.test.LoopbackMediaGroup;
import com.hp.opencall.jmsc.test.MediaSessionBase;

/**
 * Test that the methods arguments are not changed by the calls.
 * <p>Most applications will re-use the argument objects, like the RTC[]
 * and the Parameters. This requires that the implementations do not change them.
 */
public class UnchangedArgsTest extends MediaSessionBase {
	static Logger log = Logger.getLogger(UnchangedArgsTest.class);
	
	public UnchangedArgsTest() {
		super();
	}
	public UnchangedArgsTest(String testMethod) {
		super(testMethod);
	}

	/**
	 * Test that the RTC[] argument of {@link Recorder#record(URI, RTC[], Parameters)}
	 * is not altered by the call, and can be reused by the application.
	 */
	@AssertionIds( ids = { "BestPractices::arguments::2" })
	public void test_2_2_9_1_UnchangedRecordRTC() throws Exception {
		LoopbackMediaGroup myMG = getLoopbackMediaGroup(true);
		Recorder myRecorder = myMG.getRecorder();
		EventWaiter<RecorderEvent> recWaiter = new EventWaiter<RecorderEvent>(myRecorder, log);
		
		RTC[] rtcs = new RTC[] { new RTC(SignalDetector.DETECTION_OF_ONE_SIGNAL, Recorder.STOP),  MediaGroup.SIGDET_STOPPLAY };
		RTC[] orig = rtcs.clone();
		myRecorder.record(URI.create(getProperty("testcase.recordedfile")), rtcs, Parameters.NO_PARAMETER);
		recWaiter.failIfEvent(1000);
		myRecorder.stop();
		recWaiter.waitForAnyEvent(1000);
		assertEquals("RTC[] length changed", orig.length, rtcs.length);
		for (int i=0; i<orig.length; i++)
			assertEquals("RTC array entry changed", orig[i], rtcs[i]);
	}
	/**
	 * Test that the RTC[] argument of {@link SignalDetector#receiveSignals(int, Parameter[], RTC[], Parameters)}
	 * is not altered by the call, and can be reused by the application.
	 */
	@AssertionIds( ids = { "BestPractices::arguments::3" })
	public void test_2_2_9_2_UnchangedSigDetRTC() throws Exception {
		LoopbackMediaGroup myMG = getLoopbackMediaGroup(true);
		SignalDetector mySigDet = myMG.getSignalDetector();		
		EventWaiter<SignalDetectorEvent> dtmfWaiter = new EventWaiter<SignalDetectorEvent>(mySigDet, log);
		
		Parameters sigDetOpts = mySigDet.getContainer().createParameters();
		sigDetOpts.put(SignalDetector.PATTERN[0], "*");
		sigDetOpts.put(SignalDetector.PATTERN[1], "#");
		sigDetOpts.put(SignalDetector.INITIAL_TIMEOUT, 1000);

		RTC[] rtcs = new RTC[] { new RTC(SignalDetector.PATTERN_MATCH[1], SignalDetector.CANCEL),
				new RTC(SignalDetector.PATTERN_MATCH[0], SignalDetector.STOP)};
		RTC[] orig = rtcs.clone();
		mySigDet.receiveSignals(4, new Parameter[] {SignalDetector.PATTERN[0], SignalDetector.PATTERN[1]}, rtcs, sigDetOpts);
		dtmfWaiter.waitForAnyEvent(4000);
		assertEquals("RTC[] length changed", orig.length, rtcs.length);
		for (int i=0; i<orig.length; i++)
			assertEquals("RTC array entry changed", orig[i], rtcs[i]);
	}
}
