/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Copyright (c) 2009 Hewlett-Packard, Inc. All rights reserved.
 * Use is subject to license terms.
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */
package com.hp.opencall.jmsc.test.mandatory.robustness.resource;

import java.net.URI;

import javax.media.mscontrol.Parameters;
import javax.media.mscontrol.UnsupportedException;
import javax.media.mscontrol.join.Joinable.Direction;
import javax.media.mscontrol.mediagroup.MediaGroup;
import javax.media.mscontrol.mediagroup.Player;
import javax.media.mscontrol.mediagroup.signals.SignalDetector;
import javax.media.mscontrol.networkconnection.NetworkConnection;
import javax.media.mscontrol.resource.RTC;

import org.apache.log4j.Logger;

import com.hp.opencall.jmsc.test.MediaSessionBase;
/**
 * Test various error cases involving one or more {@link RTC}.
 */
public class RTCTest extends MediaSessionBase {
	static Logger log = Logger.getLogger(RTCTest.class);
	
	public RTCTest() {
		super();
	}
	public RTCTest(String testMethod) {
		super(testMethod);
	}

	/**
	 * Test that RTC(Player.PLAY_START, Player.STOP) is rejected.
	 * <br>Check in a <code>play</code>, in a <code>record<code> and in a <code>receiveSignals<code> operation.
	 */
	public void test_2_2_7_1_InvalidRTC() throws Exception {
		NetworkConnection myNC = setupaNC();
		final MediaGroup myMG = myMediaSession.createMediaGroup(MediaGroup.PLAYER_RECORDER_SIGNALDETECTOR);
		myNC.join(Direction.DUPLEX, myMG);
		try {
			myMG.getPlayer().play(URI.create(getProperty("testcase.shortpromptfile")),
				new RTC[]{ new RTC(Player.PLAY_START, Player.STOP)}, 
				Parameters.NO_PARAMETER);
			fail("RTC(Player.PLAY_START, Player.STOP) should cause UnsupportedException");
		} catch (UnsupportedException expected) {
			log.info("Caught (from Player) expected UnsupportedException: "+expected.getMessage());
		}
		try {
			myMG.getRecorder().record(URI.create(getProperty("testcase.recordedfile")),
				new RTC[]{ new RTC(Player.PLAY_START, Player.STOP)}, 
				Parameters.NO_PARAMETER);
			fail("RTC(Player.PLAY_START, Player.STOP) should cause UnsupportedException");
		} catch (UnsupportedException expected) {
			log.info("Caught (from Recorder) expected UnsupportedException: "+expected.getMessage());
		}
		try {
			myMG.getSignalDetector().receiveSignals(1, SignalDetector.NO_PATTERN,
				new RTC[]{ new RTC(Player.PLAY_START, Player.STOP)}, 
				Parameters.NO_PARAMETER);
			fail("RTC(Player.PLAY_START, Player.STOP) should cause UnsupportedException");
		} catch (UnsupportedException expected) {
			log.info("Caught (from SignalDetector) expected UnsupportedException: "+expected.getMessage());
		}
	}
	
	public void test_2_2_7_2_InvalidTriggerRTC() throws Exception {
		NetworkConnection myNC = setupaNC();
		final MediaGroup myMG = myMediaSession.createMediaGroup(MediaGroup.PLAYER);
		myNC.join(Direction.DUPLEX, myMG);
		Player myPlayer = myMG.getPlayer();
		myPlayer.play(URI.create(getProperty("testcase.shortpromptfile")), RTC.NO_RTC, Parameters.NO_PARAMETER);
		try {
			myMG.triggerAction(SignalDetector.FLUSH_BUFFER);
			fail("triggerRTC(SignalDetector.FLUSH_BUFFER) should cause Exception on a MediaGroup.PLAYER");
		} catch (Exception expected) {
			log.info("Caught expected Exception: "+expected.getMessage());
		}
	}
}
